package com.logicbig.example;

import javax.persistence.*;
import java.util.Arrays;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            nativeQuery("SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = 'PUBLIC'");
            nativeQuery("SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_SCHEMA = 'PUBLIC'");
            persistPersons();
            findAllEmployeeEntities();
            movePersonToHistoryById(1);
            findAllEmployeeEntities();
            nativeQuery("select * from PERSON_HISTORY");
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void movePersonToHistoryById(long personId) {
        System.out.printf("-- Moving person to history table id: %s --%n", personId);
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        StoredProcedureQuery procedureQuery = entityManager
                .createStoredProcedureQuery("MOVE_TO_HISTORY");
        procedureQuery.registerStoredProcedureParameter("person_id_in", Integer.class, ParameterMode.IN);
        procedureQuery.registerStoredProcedureParameter("status_out", Boolean.class, ParameterMode.OUT);
        entityManager.getTransaction().begin();
        procedureQuery.setParameter("person_id_in", (int) personId);
        procedureQuery.execute();
        Object status_out = procedureQuery.getOutputParameterValue("status_out");
        System.out.println("Out status: " + status_out);
        entityManager.getTransaction().commit();
    }

    public static void persistPersons() {
        Person person1 = Person.create("Dana", "Whitley", "464 Gorsuch Drive");
        Person person2 = Person.create("Robin", "Cash", "64 Zella Park");
        Person person3 = Person.create("Chary", "Mess", "112 Yellow Hill");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(person1);
        em.persist(person2);
        em.persist(person3);
        em.getTransaction().commit();
        em.close();
    }

    private static void findAllEmployeeEntities() {
        System.out.println("-- finding Person entities -  --");
        EntityManager em = entityManagerFactory.createEntityManager();
        TypedQuery<Person> query = em.createQuery("SELECT p from Person p", Person.class);
        List<Person> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    public static void nativeQuery(String s) {
        System.out.printf("---------------------------%n'%s'%n", s);
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createNativeQuery(s);
        List list = query.getResultList();
        for (Object o : list) {
            System.out.println(Arrays.toString((Object[]) o));
        }
    }
}